package guru.aimllib;

import processing.core.PApplet;
import bitoflife.chatterbean.*;
import bitoflife.chatterbean.parser.*;
import bitoflife.chatterbean.util.*;

import bitoflife.chatterbean.script.Interpreter;
import java.io.InputStream;
import java.io.ByteArrayOutputStream;


/**
 * @author Nikolaus Gradwohl
 * 
 * AimlLib is a Wrapper around chatterbean from <a href="http://chatterbean.bitoflife.cjb.net/">Hélio Perroni Filho</a>
 */



public class Aiml {
    private PApplet parent = null;
    private AliceBot bot = null;
    private ByteArrayOutputStream gossip = null;

    public Aiml( PApplet parent ) {
        this.parent = parent;
        parent.registerDispose(this);
        this.gossip = new ByteArrayOutputStream();
    }

    public void init( InputStream defaults, InputStream splitters, InputStream substitutions, String aimlDir, String aimlPattern ) {
        try {
            AliceBotParser parser = new AliceBotParser();
            Searcher searcher = new Searcher();
            bot = parser.parse( defaults, splitters, substitutions, searcher.search(aimlDir, aimlPattern));
            Context context = bot.getContext();
            context.outputStream( gossip );

            addToContext( "sketch", parent );
        } catch( Exception e) {
            throw new RuntimeException( "Bot Exception", e ) ;
        }
    }

    public void init( InputStream defaults, InputStream splitters, InputStream substitutions, InputStream[] brains ) {
        try {
            AliceBotParser parser = new AliceBotParser();
            Searcher searcher = new Searcher();
            bot = parser.parse( defaults, splitters, substitutions, brains );
            Context context = bot.getContext();
            context.outputStream( gossip );

            addToContext( "sketch", parent );
        } catch( Exception e) {
            throw new RuntimeException( "Bot Exception", e ) ;
        }
    }

    public void addToContext( String name, Object value ) {
        try {
            Context context = bot.getContext();
            Interpreter interpreter = (Interpreter) context.property("beanshell.interpreter");
            interpreter.variable( name, value );
        } catch ( Exception e ) {
            throw new RuntimeException( "Bot Exception", e ) ;
        }
    }

    public String getGossip() {
        String res = gossip.toString();
        gossip.reset();
        return res;
    }
    
    public String respond( String in ) {
        if (bot == null) { 
            return "not initialized";
        }
        return bot.respond( in );
    }

    public void dispose() {
    }
}

